import logging
import os
import shutil
import signal
import sys
import time
from os import path
from pathlib import Path
from socket import gethostname
from urllib.parse import urlparse

import click
import requests

from ..server import create_app, init_app
from ..services.extension_gen import ExtGen
from ..specter_error import SpecterError
from ..util.common import snake_case2camelcase
from ..util.reflection_fs import search_dirs_in_path
from ..util.shell import run_shell
from ..util.tor import start_hidden_service, stop_hidden_services

logger = logging.getLogger(__name__)

ext_mark = "specterext"
dummy_ext_url = (
    "https://raw.githubusercontent.com/cryptoadvance/specterext-dummy/master"
)


@click.group()
def ext():
    pass


@ext.command()
@click.option("--org", "org", default=None, help="Use a specific organsiation")
@click.option("--ext-id", "ext_id", default=None, help="Use a specific Extension ID")
@click.option(
    "--isolated-client/--no-isolated-client",
    default=None,
    help="Whether the extension should be isolated on the client",
)
@click.option(
    "--tmpl-fs-source",
    "tmpl_fs_source",
    help="Use a Filesystem source for the templates e.g. ~/src/specterext-dummy",
)
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="Output content on stdout instead of creating files",
)
def gen(org, ext_id, isolated_client, tmpl_fs_source, dryrun):
    # fmt: off
    """Will generate a new extension in a more or less empty directory.
    \b
    It'll ask you for the missing information if you don't pass the
    necessary details (see below).

    After creation, you can get the extension to run like this in your Development Environment:

    \b
        pip3 install -e .
        python3 -m cryptoadvance.specter server --config DevelopmentConfig --debug
        # point your browser to http://localhost:25441
        # "choose Services" --> YourService

    If you want to package it, you can build it like this:

    \b
        python3 -m pip install --upgrade build
        python3 -m build
        # install it like this:
        pip3 install dist/{org}_{ext_id}-0.0.1-py3-none-any.whl

    In order to use your extension in production, please refer to the Readme.md in the
    https://github.com/cryptoadvance/{ext_mark}-dummy#how-to-get-this-to-production

    To publish your package:

    \b
        python3 -m pip install --upgrade twine
        python3 -m twine upload --repository testpypi dist/*

    """
    # fmt: on
    if ext_id == None:
        print(
            """
            We need an ID and a prefix for your extension. It'll reflect in the package-layout.
            The id should be a short string.
            The prefix is usually something like your github-username or github organisation-name.
            Both will be used to to create the directory structure ( ./src/mycorpname/specterext/myextension )
            and it will be used to prepare the files in order to publish this extension to pypi.

        """
        )
        ext_id = click.prompt(
            "What should be the ID of your extension (lowercase only)", type=str
        )
    if org == None:
        org = click.prompt(
            "what should be the prefix?",
            type=str,
        )
    if isolated_client == None:
        print(
            """
            Should the extension be working in isolated_client-mode? In that case it's won't share the
            session-cookie with specter and the integration can only happen on server-side?
        """
        )
        isolated_client = click.prompt(
            "Should the extension work in isolated client mode (y/n)?",
            type=bool,
        )
    result = run_shell(["git", "config", "--get", "user.name"])
    if result["code"] == 0:
        author = result["out"].decode("ascii").strip()
    else:
        author = click.prompt("Please type in your Name: ", type=str)
    result = run_shell(["git", "config", "--get", "user.email"])
    if result["code"] == 0:
        email = result["out"].decode("ascii").strip()
    else:
        email = click.prompt("Please type in your E-Mail: ", type=str)

    extgen = ExtGen(
        ".",
        org,
        ext_id,
        isolated_client,
        author,
        email,
        dry_run=dryrun,
        tmpl_fs_source=tmpl_fs_source,
    )
    extgen.generate()
    # piggyback
    # replace(f"./{dir}/service.py", "piggyback = False", f"piggyback = {piggyback}")
    # if piggyback:
    #    replace(f"./{dir}/controller.py", "@login_required", "")

    print(
        f"""
        Congratulations, you've created a new extension

        Here is how to get it tor run on your Development Environment:
            pip3 install -e .
            python3 -m cryptoadvance.specter server --config DevelopmentConfig --debug
            # point your browser to http://localhost:25441
            # "choose Services" --> {ext_id}

        If you want to package it, you can build it like this:
            python3 -m pip install --upgrade build
            python3 -m build
            # install it like this:
            pip3 install dist/{org}_{ext_id}-0.0.1-py3-none-any.whl

        In order to use your extension in production, please refer to the Readme.md in the
        https://github.com/cryptoadvance/{ext_mark}-dummy#how-to-get-this-to-production
    
        To publish your package

            python3 -m pip install --upgrade twine
            python3 -m twine upload --repository testpypi dist/*

        You can get all these information again via:
        python3 -m cryptoadvance.specter ext gen --help

"""
    )



""" Here we have some constants getting an id for extension-points/callbacks. As camelcase
    is used, we don't use CAPITAL letters to not loose the meaning of the camelcase.
    
    These constants are expected as parameter to the ServiceManager.callback function
    and it'll throw an exception if the constant does not exist.

    There are some weak naming conventions:
    1. after/before
    2. class or file
    3. method or function
    
"""

afterServiceManagerInit = "afterServiceManagerInit"
after_serverpy_init_app = "after_serverpy_init_app"



import logging
from flask import Flask, Response, redirect, render_template, request, url_for
from flask import current_app as app
from flask_login import login_required, current_user

from cryptoadvance.specter.services.controller import user_secret_decrypted_required
from .service import DevhelpService
from cryptoadvance.specter.wallet import Wallet


logger = logging.getLogger(__name__)

devhelp_endpoint = DevhelpService.blueprint


@devhelp_endpoint.route("/")
@login_required
@user_secret_decrypted_required
def index():
    return render_template(
        "devhelp/index.jinja",
    )


@devhelp_endpoint.route("/html/<html_component>")
@login_required
@user_secret_decrypted_required
def html_component(html_component):
    associated_wallet: Wallet = DevhelpService.get_associated_wallet()
    return render_template(
        f"devhelp/html/{html_component}",
        wallet=associated_wallet,
        services=app.specter.service_manager.services,
        address=associated_wallet.get_address(3),
    )


@devhelp_endpoint.route("/settings", methods=["GET"])
@login_required
@user_secret_decrypted_required
def settings_get():
    associated_wallet: Wallet = DevhelpService.get_associated_wallet()

    # Get the user's Wallet objs, sorted by Wallet.name
    wallet_names = sorted(current_user.wallet_manager.wallets.keys())
    wallets = [current_user.wallet_manager.wallets[name] for name in wallet_names]

    return render_template(
        "devhelp/settings.jinja",
        associated_wallet=associated_wallet,
        wallets=wallets,
        cookies=request.cookies,
    )


@devhelp_endpoint.route("/settings", methods=["POST"])
@login_required
@user_secret_decrypted_required
def settings_post():
    show_menu = request.form["show_menu"]
    user = app.specter.user_manager.get_user()
    if show_menu == "yes":
        user.add_service(DevhelpService.id)
    else:
        user.remove_service(DevhelpService.id)
    used_wallet_alias = request.form.get("used_wallet")
    if used_wallet_alias != None:
        wallet = current_user.wallet_manager.get_by_alias(used_wallet_alias)
        DevhelpService.set_associated_wallet(wallet)
    return redirect(url_for(f"{DevhelpService.get_blueprint_name()}.settings_get"))
